import vlf4ions.sunrise_sunset as srst
import datetime as dt
from datetime import datetime
import os 


def test_sunrise_sunset():

    dirname = os.path.dirname(__file__)
    la_station = 'NWC'
    today = dt.datetime(2025, 5, 13, 14, 28, 22, 323128)
    path = os.path.join(dirname, la_station + '_sunrise_sunset.txt')

    sunrise, sunset = srst.get_sunrise_sunset(today, path)

    assert((sunrise == dt.time(3, 48)) and (sunset == dt.time(12, 47)))

def test_check_if_daytime():

    dirname = os.path.dirname(__file__)
    la_station = 'NWC'
    path = os.path.join(dirname, la_station + '_sunrise_sunset.txt')
    today = dt.datetime(2025, 5, 13, 3, 47, 0, 0)

    sunrise, sunset = srst.get_sunrise_sunset(today, path)

    flag = 0 # List of flags, 0 if in daytime, 3 in nighttime

    # Times to check
    today = dt.datetime(2025, 5, 13, 3, 48, 0, 0)
    if not srst.check_if_daytime(today, path):
        flag += 3
    today = dt.datetime(2025, 5, 13, 3, 49, 0, 0)
    if not srst.check_if_daytime(today, path):
        flag += 3
    today = dt.datetime(2025, 5, 13, 12, 46, 0, 0)
    if not srst.check_if_daytime(today, path):
        flag += 3
    today = dt.datetime(2025, 5, 13, 12, 47, 0, 0)
    if not srst.check_if_daytime(today, path):
        flag += 3


    assert(flag == 6)  


    






#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Wed Aug 21 10:44:38 2024

@author: pteysseyre
"""

import numpy as np
import vlf4ions.sunrise_sunset as srst

# History
# ---------------------


def flags(amp_NS, amp_EW, previous_nb_points, today, path):
    """Flags 1 if the antenna is down, 2 if the transmitter is down, 3 if it is nighttime"""
    
    flag = 0
    data_last_min = 0
    length_data = min(len(amp_NS), len(amp_EW)) # this is necessary as there may be one more point in amp_EW than amp_NS, as it is read later
    
    #Check that we are in daytime
    daytime = srst.check_if_daytime(today, path)
    
    if daytime:
        # Antenna down: the number of data points in the last written file stays the same
        if length_data == previous_nb_points:
            flag = 1
        else:
            previous_nb_points = length_data
        
            # Transmitter down: the amplitude is less than 0.3 pT
            
            if length_data > 60:
                amp_NS = amp_NS[-60:]
                amp_EW = amp_EW[-60:]
            else:
                amp_NS = amp_NS[-length_data:]
                amp_EW = amp_EW[-length_data:]
                
            amp = np.sqrt(np.square(amp_NS) + np.square(amp_EW))
            data_last_min = np.median(amp)
        
        if data_last_min < 0.3: # 0.3 pT is the antenna sensibility
            flag = 2
    else:
        flag = 3
        
    return flag, length_data, data_last_min
    


#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Tue Feb 11 14:44:17 2025

@author: pteysseyre
"""

# CAREFUL, that works !

import smtplib
from email.mime.text import MIMEText  

def send_email(reason, recipients, receiver):
    
    ''' This is the function called outside of this file, but really it only sends a message'''
    
    subject, body, sender, password = _get_mail_content(receiver, reason)
    
    msg = MIMEText(body)
    msg['Subject'] = subject
    msg['From'] = sender
    msg['To'] = ', '.join(recipients)
    
    with smtplib.SMTP_SSL('smtp.gmail.com', 465) as smtp_server:
       smtp_server.login(sender, password)
       smtp_server.sendmail(sender, recipients, msg.as_string())
    print("Message sent!")


def _get_mail_content(receiver, reason):
    
    '''This function takes into account a reason to send a message and decides on the contents
    
    Parameters:
    ------------------------------
    - reason: If the antenna is down, 'reason' = 'Antenna down'. 'Error' is only used for testing, to send me a message when there was a problem reading the file or such.
    
    TODO: So far, it only alerts if the antenna is down. But it actually needs to give alerts & estimations in case of flares !
    Note : Maybe not all users will be interested in the same alerts.
    
    '''
    
    sender = 'ptey.secondary@gmail.com'
    password = 'ewkl zcfi qcjb wxdx'
    
    if reason == 'Antenna down':
        
        subject = 'Antenna down'
        body = receiver, 'antenna is down - The recording should be checked and launched again if needed'
    
    if reason == 'Error':
        
        subject = 'Error in ' + receiver
        body = 'Check the log file, there was an error in the file reading/breakpoint analysis'
    
    return(subject, body, sender, password)


        
        
def send_alerts(la_station, receiver, time_now, p1, recipients):

        subject = 'Detection in ' + receiver + ' - ' + la_station
        sender = 'ptey.secondary@gmail.com'
        password = 'ewkl zcfi qcjb wxdx'
        
        body = 'Detection at ' + str(time_now) + ' UT \n Slope : ' + str(p1) + ' deg/hr.'
        
        msg = MIMEText(body)
        msg['Subject'] = subject
        msg['From'] = sender
        msg['To'] = ', '.join(recipients)
        
        with smtplib.SMTP_SSL('smtp.gmail.com', 465) as smtp_server:
           smtp_server.login(sender, password)
           smtp_server.sendmail(sender, recipients, msg.as_string())
        print("Message sent!")
        
        return

def send_midnightping(receiver, recipients):

        subject = 'Midnight check for ' + receiver
        sender = 'ptey.secondary@gmail.com'
        password = 'ewkl zcfi qcjb wxdx'
        
        body = 'The real time script is still running'
        
        msg = MIMEText(body)
        msg['Subject'] = subject
        msg['From'] = sender
        msg['To'] = ', '.join(recipients)
        
        with smtplib.SMTP_SSL('smtp.gmail.com', 465) as smtp_server:
           smtp_server.login(sender, password)
           smtp_server.sendmail(sender, recipients, msg.as_string())
        print("Message sent!")
        
        return


#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Tue Apr 22 14:10:40 2025

@author: pteysseyre
"""

import vlf4ions.read_files as rf
import vlf4ions.detect_change_points as dcp
import vlf4ions.correct_phase as correct_phase
import vlf4ions.flag as fl
import vlf4ions.analyse_change_points as acp
import vlf4ions.manage_statefile as msf

import numpy as np
import datetime as dt # Necessary to use timezone
from datetime import datetime
import logging

# For alerts
import smtplib
from email.mime.text import MIMEText  


#----------------------- Receiver description -----------------
class receiver:
    ''' Characteristics of the receiver (for plotting purposes mainly and for sending alerts) '''
    def __init__(self, name, lat, lon):
        self.name = name
        self.lat = lat
        self.lon = lon

#---------------------- Alerts, warnings and such --------------
class email_alerts: 
    """ Alerts sent in case of flare detection, antenna problems or such """
    
    def __init__(self, subject, body, sender, password, recipients):
        self.subject = subject
        self.body = body
        self.sender = sender
        self.password = password
        self.recipients = recipients
    
    def update_detection_body(self, callsign, p1):
        self.body = 'Detection with ' + callsign + '\n Slope : ' + str(p1) + ' deg/hr.'
        
    def send_email(self): 
        
        msg = MIMEText(self.body)
        msg['Subject'] = self.subject
        msg['From'] = self.sender
        msg['To'] = ', '.join(self.recipients)
        
        with smtplib.SMTP_SSL('smtp.gmail.com', 465) as smtp_server:
           smtp_server.login(self.sender, self.password)
           smtp_server.sendmail(self.sender, self.recipients, msg.as_string())  

#------------------- Stations and related functions --------------

def sliding_median(arr, window):
    return np.median(np.lib.stride_tricks.sliding_window_view(arr, (window,)), axis=1)
   
class station:
    ''' Characteristics of the transmitter.
    
    ---------------
    Params:
    - name: call-sign (e.g. 'GVT')
    - lat/lon: latitude and longitude of the transmitter
    - freq: Frequency (kHz)
    - detection_threshold: Phase slope above which we detect a solar flare
    - reading_time: second after the minute at which we read the data. NOTE: They should be unique for each station
    - time_average: Time resolution of the smoothed signal (default: 60 s)
    - delta: Delta parameter of the incremental algorithm (Guralnik & Srivastava, 1999. Default:0.1)
    - nb_data_points: number of data points last time the file was read (default : 0, used for flagging)
    - nb_error: consecutive number of errors in the file reading (default : 0, used for flagging)
    - Ne: Average electron density computed in the path (not done yet, but I needed this for data analysis)
    
    ---------------
    History: 
        Written by P. Teysseyre in January 2025
        Re-written under current format in April 2025
    
    
    
    '''
    def __init__(self, name, lat, lon, freq, detection_threshold, reading_time, time_average = 60, delta = 0.1, nb_data_points = 0, nb_error = 0, Ne=1e8):
        self.name = name
        self.lat = lat
        self.lon = lon
        self.freq = freq
        self.detection_threshold = detection_threshold
        self.reading_time = reading_time
        self.time_average = time_average
        self.delta = delta
        self.nb_data_points = nb_data_points
        self.nb_error = nb_error
        self.Ne = Ne
    
    
    def update_breakpoints(self, path_breakpoint, path_mau, path_sunrise, alerts_antenna, alerts_detection):
        
        try:
            # Get today's date in a good format
            today = dt.datetime.now(dt.timezone.utc) # The time zone is necessary to use get_sza later
            la_date = today.strftime("%Y_%m_%d")
            
            # Read data & corrects the phase
            amp_NS, amp_EW, phase_NS, phase_EW, the_time, nb_error = rf.read_Narrowband_real_time(la_date, self.name, path_mau)
            phase, jumps = correct_phase.correctionphaseparDw(self.name, phase_EW, the_time)
            
            # Flag the data if needed
            # CAREFUL: amp is the median of the amplitude recorded over the last minute at most
            path_sunrise = path_sunrise +self.name+'_sunrise_sunset.txt'
            flag, nb_data_points, amp = fl.flags(amp_NS, amp_EW, self.nb_data_points, today, path_sunrise)
            self.nb_data_points = nb_data_points
            
            if flag == 1: # The antenna has stopped
                self.nb_error += 1
                
                
            elif flag == 0:
                
                self.nb_error = 0
                
                # Time average the data & apply median filter
                time_average = np.min([self.time_average, len(the_time)])
                the_time = sliding_median(the_time, self.time_average)
                the_time = the_time[0::time_average]
                phase = sliding_median(phase, self.time_average)
                phase = phase[0::time_average]
                
                # Read the last breakpoint value OR create a new file if it is the first of the day
                last_breakpoint, last_p1, last_amp, last_phase, quiet, DP = msf.get_lastbreakpoint(self.name, la_date, path_breakpoint)
                
                # Keep the data after last breakpoint
                mask_time = (the_time > last_breakpoint)
                the_time = the_time[mask_time]
                phase = phase[mask_time]
                
                try :
                    # Shorten the data if it is too long
                    if the_time[-1] - the_time[0] > 2:
                        two_hours = int(np.floor(7200/self.time_average))
                        phase = phase[-1 - two_hours:]
                        the_time = the_time[-1 - two_hours:]
                        
                except :
                    self.nb_error += 1
            
                # Look if there are new breakpoints now
                detec, p1 = dcp.find_break_point_realtime(the_time, phase, self.delta)
            
                # If there is a new breakpoint, write it in a file
                if detec:   
                
                    # Get last QUIET breakpoint
                    quiet_breakpoint, quiet_p1, quiet_amp, quiet_phase, quiet_quiet, quiet_DP = msf.get_lastbreakpoint(self.name, la_date, path_breakpoint, quiet=True)
                    # NOTE: quiet_quiet is useless and is always 1 (same for quiet_DP who is always 0)
                    
                    quiet_now, prev_1min, prev_5min, prev_timedecay, DP = acp.analyse_breakpoint(the_time[-1], p1, self.name, amp, phase, the_time, quiet, quiet_phase, quiet_amp, last_breakpoint, last_p1, self.detection_threshold, DP)
                    
                    # Write new breakpoint in file
                    msf.write_newbreakpoint(self.name, la_date, path_breakpoint, the_time[-1], p1, amp, phase[-1], quiet_now, DP)
                    
                    print('New breakpoint !')
                    
                    # Send alerts if needed
                    if quiet_now == 0 and quiet == 1:
                        
                        alerts_detection.update_detection_body(self.name, p1)
                        alerts_detection.send_email()
                
                print('Done -  ', today.strftime("%H:%M:%S"), '-', self.name)
            
                
            elif flag == 3:
                
                print(self.name, ' at ', today.strftime("%H:%M:%S"), ' : nighttime')
                
            else : 
                
                print(self.name, ' is not transmitting at ', today.strftime("%H:%M:%S"))
        
                
        except Exception as e: # If there was an error, we update the log file
        
            self.nb_error += 1
            
            # Configuration of the logging file
            logging.basicConfig(filename='detection.log', encoding='utf-8')
            logging.error('Error at %s', 'division', exc_info=e)
            print(self.name, ' - error at ', today.strftime("%H:%M:%S"), ' - see log')
        
        if self.nb_error == 5: # it's been five minutes since the antenna stopped or had problems
            
            alerts_antenna.send_email()
            
            


#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Fri Feb  7 14:28:49 2025

@author: pteysseyre
"""

import numpy as np
import vlf4ions.compute_electron_density as cne


def analyse_breakpoint(current_time, current_p1, la_station, amp, phase, the_time, quiet, quiet_phase, quiet_amp, last_time, last_p1, detection_threshold, DP):

    """This is the part of the code that will decide if the  breakpoint is a flare signature or not.

    Several cases can occur:

        1) This is the beginning of a flare. In this case, we can estimate the current X-ray strength and electron density
        by taking as a reference the last 'quiet' breakpoint. We can also estimate the state of the ionosphere in 1 and 5 min
        assuming that the trend doesn't change

        2) This is the first brekapoint to signal a flare decay. We can estimate the current X-ray flux, electron density, and
        give an estimate of when things will have decreased to half of their increase

        3) This is a quiet point: this is either a slight positive slope, a negative slope after a quiet breakpoint, or a negative
        slope after a flare has decayed by more than half of its increase

        4) This is a flare decay, but this is not the first detection and the flare has not gone down by more than half of its
        increase. We can update the estimated X-ray flux and Ne, and estimate the new time to the end of the flare

        Parameters:
        ---------------------------

        - la_station, phase: the transmitter, recorded & corrected phase (in °) since beginning of file
        - amp: median over the last minute of amplitude data.
        - detection_threshold: Standard deviation of slopes found in quiet times
        - the_time: time-array. Dimension should match  phase
        - pathtoCSVfiles: path to access the CSV files obtained with the LMP model
        - current_p1, current_time : current breakpoint slope detected and time  of detection
        - last_p1 , last_time: last  breakpoint slope detected and time of detection
        - quiet: is True if the last brealpoint detected was a quiet breakpoint. Is False otherwise (flares)
        - quiet_phase, quiet_amp: last read phase & amp (to avoid reading it and looking for it)

        Outputs:
        ---------------------------

        - quiet_now : is True if the current breakpoint is in quiet time, False otherwise
        - flux : estimated X-ray flux and confidence interval
        - prev_1min : estimated Ne in 1 min
        - prev_5min : estimated Ne in 5 min
        - prev_timedecay : estimated time of decay


        IMPORTANT NOTE:
        --------------------------
        1) Right now (Feb. 2025), we are concentrating on the phase (Ne in code is phase).
        However, we can for a second step concentrate on the electron density at an altitude of 70 km;
        moreover, the interesting quantity for us is the HF absorption. Thus, in the future, the HF absorption should be
        computed and the time of the flare's decay, as well as the previsions for the next minutes or so should be based
        on this quantity instead.

        2) Right now (Feb. 2024), the quiet electron density is estimated to be given by H = 74 km and B = 0.3 km^-1 in
        Wait's model. This can easily be improved upon by using different values from the works by Thomson for example.

        Documentation and diagrams:
        ----------------------------
        See project vlf4ions on GitLab, 'Redaction.zip' for the full documentation """

    ### TODO: Think about the flares happening in a very fast sucession

    # Initialisation: most of what is returned will not be systematically useful
    prev_timedecay, prev_1min, prev_5min = np.zeros(3)
    quiet_now = 0

    DP_now = phase[-1] - quiet_phase
    
    if current_p1 > 0:

        if (current_p1 < detection_threshold):
            
            if (quiet == 1): #This is a quiet time
                quiet_now = 1
                DP = 0
                
            else : # That's either a flare max OR a flare that has decayed
                
                # We look for the phase maximum to get DP
                the_time, phase = _keep_between(phase, the_time, last_time, current_time)
                ind_max = np.argmax(phase)
                DP = phase[ind_max] - quiet_phase
                time_max = the_time[ind_max]
                
                if DP_now < DP/2: # The flare has already decayed
                    quiet_now = 1
                    DP = 0
                else : # This is the max of the flare
                    quiet_now = 0
                    prev_1min = DP
                    prev_5min = DP

        else: # This is a flare

            # TODO: change for Ne or abs
            DA = amp - quiet_amp

            # Compute Ne here
            # TODO: Estimate X-ray flux here

            # Estimate phase in 1 or 5 min
            prev_1min = current_p1*1/60 + phase[-1]
            prev_5min = current_p1*5/60 + phase[-1]


    else: # This is either quiet time, or a flare's decay

        if quiet: # Last breakpoint is quiet
            quiet_now = True

        else: # Last breakpoint was a flare decay

            if last_p1 > 0: # Last breakpoint was a flare onset

                ### TODO: Use Ne instead of phase
                
                # We look for the phase maximum to get DP
                the_time, phase = _keep_between(phase, the_time, last_time, current_time)
                ind_max = np.argmax(phase)
                DP = phase[ind_max] - quiet_phase
                time_max = the_time[ind_max]

                # We estimate the decay time assuming that the trend is constant & the decay linear
                prev_timedecay = -DP/(2*current_p1) + time_max

            else :

                # Check if the flare has decayed (lost more than half its increase)
                ### TODO: change for Ne or abs

                if DP_now < DP/2: # The flare has ended
                    quiet_now = True

                else: # The flare is still ongoing but we can update our previsions
                    prev_timedecay = 1/current_p1*(DP/2 - DP_now) + the_time[-1]



    return (quiet_now, prev_1min, prev_5min, prev_timedecay, DP)



def _keep_between(data, the_time, time_inf, time_sup):

    """ This is just a short bit of code to constrain the data between time_inf & time_sup """

    if len(data) != len(the_time):

        print('Careful, lengths do not match - keep_between')

    else:

        ind_inf = np.argmin(np.abs(the_time - time_inf))
        ind_sup = np.argmin(np.abs(the_time - time_sup))

        the_time = the_time[ind_inf:ind_sup]
        data = data[ind_inf:ind_sup]

        return the_time, data












